# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius``"""

import builtins as builtins
import collections.abc as abc
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message
import nebius.api.nebius.annotations_pb2 as annotations_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import typing as typing
#@ local imports here @#

# file: nebius/annotations.proto
class ResourceBehavior(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.ResourceBehavior",annotations_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    RESOURCE_BEHAVIOR_UNSPECIFIED = 0
    """
    The behavior of the resource is unspecified.
    Avoid using this default value.
    """
    
    MOVABLE = 1
    """
    Indicates that the resource can be moved to another parent, typically an
    IAM container, though not necessarily limited to this.
    This behavior suggests that the ``metadata.parent_id`` attribute could be modified.
    """
    
    UNNAMED = 2
    """
    Indicates that the resource name can be unspecified or does not follow
    uniqueness requirement within parent_id and resource type.
    """
    
    IMMUTABLE_NAME = 3
    """
    Indicates that the resource is named, and the name cannot be changed after
    it is created. It is strongly recommended to do srvices with renaming
    capability, as the guidelines suggest.
    """
    

class FieldBehavior(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.FieldBehavior",annotations_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    FIELD_BEHAVIOR_UNSPECIFIED = 0
    IMMUTABLE = 2
    """
    This indicates that the field can't be changed during a resource update.
    Changing the field value will cause an ``INVALID_ARGUMENT`` error.
    For message fields, this applies to the field itself. Nested immutable
    fields do not make a mutable parent message immutable, and such a parent
    message may still be cleared.
    Resource recreate requires a change of the field value.
    """
    
    IDENTIFIER = 3
    """
    Indicates field is a resource ID, so it MUST be present on a resource
    update, but MUST NOT be set on create.
    Otherwise, RPC will fail with the ``INVALID_ARGUMENT`` error
    """
    
    INPUT_ONLY = 4
    """
    Indicates field is not present in output.
    """
    
    OUTPUT_ONLY = 5
    """
    Indicates field can't be set on create or changed on update.
    Otherwise, RPC will fail with the ``INVALID_ARGUMENT`` error
    """
    
    MEANINGFUL_EMPTY_VALUE = 6
    """
    Indicates that an empty message and a null have different semantics.
    Usually, that field is a feature spec message: its empty message enables
    that feature, and null disables it. Such a message is different from ``bool``
    because it already has some feature parameters, or they can be added later
    in a backward-compatible way.
    IMPORTANT: if the message itself is recursive, this behavior is forced.
    """
    
    NON_EMPTY_DEFAULT = 7
    """
    Indicates that an empty (default) value will be filled by the server.
    Usually, that field is a feature spec value, which by default is computed.
    Values marked with this annotation won't raise error if they are not set
    and the returned value is not equal to protobuf default.
    
    IMPORTANT:
    Updating this value from explicit to default may not lead to Update call in
    some tools (eg Terraform).
    Compound values (messages, lists and maps) may result in unpredictable
    updates (see examples in guidelines).
    """
    

class MethodBehavior(pb_enum.Enum):
    """
    MethodBehavior describes special behaviors of a method that affect
    code generation and tooling.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.MethodBehavior",annotations_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    METHOD_BEHAVIOR_UNSPECIFIED = 0
    """
    Indicates that the method behavior is default and is not specified.
    For instance, an Update method will lose its update semantics if this value
    is set.
    Does not mean anything if set along with other values.
    """
    
    METHOD_UPDATER = 2
    """
    Indicates that the method is used to update a resource or a number of
    resources, therefore it requires a reset mask to unset fields.
    This will enable SDKs to generate the reset mask on request, as well as the
    gateway to pass and sanitize it. The CLI will add flags to set partial or
    full updates and custom reset masks.
    """
    
    METHOD_PAGINATED = 3
    """
    Indicates that the method is used to list something, and supports
    pagination.
    """
    
    METHOD_WITHOUT_GET = 4
    """
    Indicates that the method will create an operation, which doesn't have a
    gettable resource after its completion, and therefore must not call a ``get``
    method. For instance, it can be used for long-running operations that
    don't return any resource, or for resource deletion operations.
    """
    

class ServicePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.ServicePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.ServicePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class MethodPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.MethodPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.MethodPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class FieldPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.FieldPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.FieldPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class MessagePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.MessagePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.MessagePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class OneofPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.OneofPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.OneofPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class EnumPySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.EnumPySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.EnumPySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class EnumValuePySDKSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.EnumValuePySDKSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.EnumValuePySDKSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        name: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(name, unset.UnsetType):
            self.name = name
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "name",
        ]
    
    @builtins.property
    def name(self) -> "builtins.str":
        return super()._get_field("name", explicit_presence=False,
        )
    @name.setter
    def name(self, value: "builtins.str|None") -> None:
        return super()._set_field("name",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "name":"name",
    }
    
class DeprecationDetails(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.DeprecationDetails
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.DeprecationDetails",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        effective_at: "builtins.str|None|unset.UnsetType" = unset.Unset,
        description: "builtins.str|None|unset.UnsetType" = unset.Unset,
        description_cli: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(effective_at, unset.UnsetType):
            self.effective_at = effective_at
        if not isinstance(description, unset.UnsetType):
            self.description = description
        if not isinstance(description_cli, unset.UnsetType):
            self.description_cli = description_cli
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "effective_at",
            "description",
            "description_cli",
        ]
    
    @builtins.property
    def effective_at(self) -> "builtins.str":
        """
        The date when this method, service, message or field will stop working (format: YYYY-MM-DD)
        """
        
        return super()._get_field("effective_at", explicit_presence=False,
        )
    @effective_at.setter
    def effective_at(self, value: "builtins.str|None") -> None:
        return super()._set_field("effective_at",value,explicit_presence=False,
        )
    
    @builtins.property
    def description(self) -> "builtins.str":
        """
        A description to help users understand the reason for deprecation and suggest alternatives.
        By default, this message will be shown in the CLI, Terraform, and SDKs as is.
        """
        
        return super()._get_field("description", explicit_presence=False,
        )
    @description.setter
    def description(self, value: "builtins.str|None") -> None:
        return super()._set_field("description",value,explicit_presence=False,
        )
    
    @builtins.property
    def description_cli(self) -> "builtins.str":
        """
        Override base description for CLI, specify possible alternative method
        Example: "Please use the 'iam v2 access-key' command instead. All existing keys remain accessible through the new command."
        """
        
        return super()._get_field("description_cli", explicit_presence=False,
        )
    @description_cli.setter
    def description_cli(self, value: "builtins.str|None") -> None:
        return super()._set_field("description_cli",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "effective_at":"effective_at",
        "description":"description",
        "description_cli":"description_cli",
    }
    
class NIDFieldSettings(pb_classes.Message):
    __PB2_CLASS__ = annotations_pb2.NIDFieldSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.NIDFieldSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        resource: "abc.Iterable[builtins.str]|None|unset.UnsetType" = unset.Unset,
        parent_resource: "abc.Iterable[builtins.str]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(resource, unset.UnsetType):
            self.resource = resource
        if not isinstance(parent_resource, unset.UnsetType):
            self.parent_resource = parent_resource
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "resource",
            "parent_resource",
        ]
    
    @builtins.property
    def resource(self) -> "abc.MutableSequence[builtins.str]":
        """
        Fields annotated with this option are treated as NIDs.
        ``resource`` lists allowed NID resource types (prefixes). Leave empty or set to ``*`` to accept any type.
        Validation only produces warnings.
        """
        
        return super()._get_field("resource", explicit_presence=False,
        wrap=pb_classes.Repeated,
        )
    @resource.setter
    def resource(self, value: "abc.Iterable[builtins.str]|None") -> None:
        return super()._set_field("resource",value,explicit_presence=False,
        )
    
    @builtins.property
    def parent_resource(self) -> "abc.MutableSequence[builtins.str]":
        """
        For metadata fields, ``parent_resource`` lists allowed parent resource types for ``metadata.parent_id``.
        Leave empty or set to ``*`` to allow any type. Validation only produces warnings.
        Typically set on the resource message; request-level overrides are supported.
        """
        
        return super()._get_field("parent_resource", explicit_presence=False,
        wrap=pb_classes.Repeated,
        )
    @parent_resource.setter
    def parent_resource(self, value: "abc.Iterable[builtins.str]|None") -> None:
        return super()._set_field("parent_resource",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "resource":"resource",
        "parent_resource":"parent_resource",
    }
    
class SubfieldSettings(pb_classes.Message):
    """
    SubfieldSettings describes overrides for some settings for subfields of a
    field. Overrides are applied to fields in order of appearance, so the first
    matching override is applied, and the rest are ignored.
    
    Example:
    
    .. code-block:: protobuf
    
       message MyMessage {
         string field1 = 1;
       }
       message MyMessage2 {
         MyMessage field2 = 1 [(subfield_settings) = { field_path: "field1", is_required: true }];
       }
    
    In this example, ``field1`` in ``MyMessage2`` is required, even if it is not
    required in ``MyMessage``.
    The following example will override the setting again:
    
    .. code-block:: protobuf
    
       service MyService {
         rpc MyMethod(MyMessage2) returns (MyMessage2) {
           option (method_behavior) = METHOD_UPDATER;
           option (request_fields) = { field_path: "field2.field1", is_required: false };
         }
       }
    
    In this example, ``field1`` in ``MyMessage2`` is not required for the ``MyMethod``,
    even if it is required in ``MyMessage2``.
    """
    
    __PB2_CLASS__ = annotations_pb2.SubfieldSettings
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.SubfieldSettings",annotations_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class __OneOfClass__is_required__(pb_classes.OneOf):
        name: builtins.str= "_is_required"
        
        def __init__(self, msg: "SubfieldSettings") -> None:
            super().__init__()
            self._message: "SubfieldSettings" = msg
    
    class __OneOfClass__is_required_is_required__(__OneOfClass__is_required__):
        field: typing.Literal["is_required"] = "is_required"
        
        def __init__(self, msg: "SubfieldSettings") -> None:
            super().__init__(msg)
        @builtins.property
        def value(self) -> "builtins.bool":
            return self._message.is_required
    
    @builtins.property
    def _is_required(self) -> __OneOfClass__is_required_is_required__|None:
        field_name: str|None = super().which_field_in_oneof("_is_required")
        match field_name:
            case "is_required":
                return self.__OneOfClass__is_required_is_required__(self)
            case None:
                return None
            case _:
                raise pb_classes.OneOfMatchError(field_name)
    
    def __init__(
        self,
        initial_message: message.Message|None = None,
        *,
        field_path: "builtins.str|None|unset.UnsetType" = unset.Unset,
        nid: "NIDFieldSettings|annotations_pb2.NIDFieldSettings|None|unset.UnsetType" = unset.Unset,
        is_required: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(field_path, unset.UnsetType):
            self.field_path = field_path
        if not isinstance(nid, unset.UnsetType):
            self.nid = nid
        if not isinstance(is_required, unset.UnsetType):
            self.is_required = is_required
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "field_path",
            "nid",
            "is_required",
            "_is_required",
        ]
    
    @builtins.property
    def field_path(self) -> "builtins.str":
        """
        Subfield path in the message, for example ``metadata.parent_id`` or ``metadata.name``.
        Must be a valid SelectMask.
        May match several fields, if necessary. For example, ``some.*.subfield`` or
        ``some.(subfield,another_field)``.
        """
        
        return super()._get_field("field_path", explicit_presence=False,
        )
    @field_path.setter
    def field_path(self, value: "builtins.str|None") -> None:
        return super()._set_field("field_path",value,explicit_presence=False,
        )
    
    @builtins.property
    def nid(self) -> "NIDFieldSettings":
        """
        For fields annotated with this option, values are treated as NIDs and
        warnings are emitted when they are not valid. These warnings are separate from server-side validation.
        """
        
        return super()._get_field("nid", explicit_presence=False,
        wrap=NIDFieldSettings,
        )
    @nid.setter
    def nid(self, value: "NIDFieldSettings|annotations_pb2.NIDFieldSettings|None") -> None:
        return super()._set_field("nid",value,explicit_presence=False,
        )
    
    @builtins.property
    def is_required(self) -> "builtins.bool|None":
        """
        Mark a field as required, even if it is not required in the message it contains.
        Unlike cel expressions, this setting can be reflected in tools documentation and
        help messages.
        """
        
        return super()._get_field("is_required", explicit_presence=True,
        )
    @is_required.setter
    def is_required(self, value: "builtins.bool|None") -> None:
        return super()._set_field("is_required",value,explicit_presence=True,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "field_path":"field_path",
        "nid":"nid",
        "is_required":"is_required",
        "_is_required":"_is_required",
    }
    
file_deprecation_details = annotations_pb2.file_deprecation_details
api_service_name = annotations_pb2.api_service_name
service_deprecation_details = annotations_pb2.service_deprecation_details
service_py_sdk = annotations_pb2.service_py_sdk
method_deprecation_details = annotations_pb2.method_deprecation_details
method_py_sdk = annotations_pb2.method_py_sdk
method_behavior = annotations_pb2.method_behavior
request_fields = annotations_pb2.request_fields
resource_behavior = annotations_pb2.resource_behavior
message_deprecation_details = annotations_pb2.message_deprecation_details
message_py_sdk = annotations_pb2.message_py_sdk
field_behavior = annotations_pb2.field_behavior
sensitive = annotations_pb2.sensitive
credentials = annotations_pb2.credentials
field_deprecation_details = annotations_pb2.field_deprecation_details
field_py_sdk = annotations_pb2.field_py_sdk
nid = annotations_pb2.nid
subfield_settings = annotations_pb2.subfield_settings
oneof_behavior = annotations_pb2.oneof_behavior
oneof_py_sdk = annotations_pb2.oneof_py_sdk
enum_py_sdk = annotations_pb2.enum_py_sdk
enum_deprecation_details = annotations_pb2.enum_deprecation_details
enum_value_deprecation_details = annotations_pb2.enum_value_deprecation_details
enum_value_py_sdk = annotations_pb2.enum_value_py_sdk
__all__ = [
    #@ local import names here @#
    "ResourceBehavior",
    "FieldBehavior",
    "MethodBehavior",
    "file_deprecation_details",
    "api_service_name",
    "service_deprecation_details",
    "service_py_sdk",
    "method_deprecation_details",
    "method_py_sdk",
    "method_behavior",
    "request_fields",
    "resource_behavior",
    "message_deprecation_details",
    "message_py_sdk",
    "field_behavior",
    "sensitive",
    "credentials",
    "field_deprecation_details",
    "field_py_sdk",
    "nid",
    "subfield_settings",
    "oneof_behavior",
    "oneof_py_sdk",
    "enum_py_sdk",
    "enum_deprecation_details",
    "enum_value_deprecation_details",
    "enum_value_py_sdk",
    "ServicePySDKSettings",
    "MethodPySDKSettings",
    "FieldPySDKSettings",
    "MessagePySDKSettings",
    "OneofPySDKSettings",
    "EnumPySDKSettings",
    "EnumValuePySDKSettings",
    "DeprecationDetails",
    "NIDFieldSettings",
    "SubfieldSettings",
]



# type: ignore
from __future__ import annotations

from datetime import datetime, timedelta, timezone

import pytest


@pytest.mark.asyncio
async def test_operation_progress_tracker_updates() -> None:
    import grpc.aio
    from google.protobuf.timestamp_pb2 import Timestamp
    from google.rpc.status_pb2 import Status as StatusPb

    import nebius.api.nebius.common.v1.operation_pb2 as operation_pb2
    import nebius.api.nebius.common.v1.progress_tracker_pb2 as progress_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.aio.operation import Operation
    from nebius.api.nebius.common.v1.operation_service_pb2_grpc import (
        OperationServiceServicer,
        add_OperationServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE
    from nebius.base.protos.well_known import local_timezone

    def ts(dt: datetime) -> Timestamp:
        stamp = Timestamp()
        stamp.FromDatetime(dt.astimezone(timezone.utc))
        return stamp

    def to_local(dt: datetime) -> datetime:
        return dt.astimezone(local_timezone)

    def op_with_tracker(
        tracker: progress_pb2.ProgressTracker | None = None,
        *,
        status: StatusPb | None = None,
        finished_at: datetime | None = None,
    ) -> operation_pb2.Operation:
        op = operation_pb2.Operation(id="op-1")
        if tracker is not None:
            op.progress_tracker.CopyFrom(tracker)
        if status is not None:
            op.status.CopyFrom(status)
        if finished_at is not None:
            op.finished_at.CopyFrom(ts(finished_at))
        return op

    base_now = datetime.now(timezone.utc)
    started_past = base_now - timedelta(seconds=120)
    started_now = base_now - timedelta(seconds=5)
    started_future = base_now + timedelta(seconds=60)
    estimate_future = base_now + timedelta(seconds=120)
    estimate_past = base_now - timedelta(seconds=10)
    estimate_updated = base_now + timedelta(seconds=30)
    finished_time = base_now + timedelta(seconds=1)

    op0 = operation_pb2.Operation(id="op-1")

    op1 = op_with_tracker(progress_pb2.ProgressTracker())

    op2 = op_with_tracker(
        progress_pb2.ProgressTracker(
            description="phase-1",
            started_at=ts(started_now),
            work_done=progress_pb2.ProgressTracker.WorkDone(
                total_tick_count=10,
                done_tick_count=2,
            ),
        )
    )

    op3 = op_with_tracker(
        progress_pb2.ProgressTracker(
            description="phase-1-est",
            started_at=ts(started_future),
            estimated_finished_at=ts(estimate_future),
        )
    )

    step_a = progress_pb2.ProgressTracker.Step(
        description="step-a",
        started_at=ts(started_past),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=4,
            done_tick_count=1,
        ),
    )
    step_b = progress_pb2.ProgressTracker.Step(
        description="step-b",
        started_at=ts(started_past),
    )
    tracker4 = progress_pb2.ProgressTracker(
        description="phase-2",
        started_at=ts(started_past),
        estimated_finished_at=ts(estimate_past),
    )
    tracker4.steps.extend([step_a, step_b])
    op4 = op_with_tracker(tracker4)

    step_a_updated = progress_pb2.ProgressTracker.Step(
        description="step-a",
        started_at=ts(started_past),
        finished_at=ts(base_now - timedelta(seconds=5)),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=4,
            done_tick_count=4,
        ),
    )
    step_b_updated = progress_pb2.ProgressTracker.Step(
        description="step-b updated",
        started_at=ts(started_past),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=2,
            done_tick_count=1,
        ),
    )
    tracker5 = progress_pb2.ProgressTracker(
        description="phase-2 updated",
        started_at=ts(started_past),
        estimated_finished_at=ts(estimate_updated),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=10,
            done_tick_count=7,
        ),
    )
    tracker5.steps.extend([step_a_updated, step_b_updated])
    op5 = op_with_tracker(tracker5)

    step_a_done = progress_pb2.ProgressTracker.Step(
        description="step-a",
        started_at=ts(started_past),
        finished_at=ts(finished_time),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=4,
            done_tick_count=4,
        ),
    )
    step_b_done = progress_pb2.ProgressTracker.Step(
        description="step-b updated",
        started_at=ts(started_past),
        finished_at=ts(finished_time),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=2,
            done_tick_count=2,
        ),
    )
    tracker6 = progress_pb2.ProgressTracker(
        description="done",
        started_at=ts(started_past),
        finished_at=ts(finished_time),
        work_done=progress_pb2.ProgressTracker.WorkDone(
            total_tick_count=10,
            done_tick_count=10,
        ),
    )
    tracker6.steps.extend([step_a_done, step_b_done])
    op6 = op_with_tracker(tracker6, status=StatusPb(code=0), finished_at=finished_time)

    class MockOperationService(OperationServiceServicer):
        def __init__(self, responses: list[operation_pb2.Operation]) -> None:
            self._responses = responses
            self._index = 0

        async def Get(self, request, context):  # noqa: N802
            assert request.id == "op-1"
            if self._index < len(self._responses):
                response = self._responses[self._index]
                self._index += 1
                return response
            return self._responses[-1]

    srv = grpc.aio.server()
    port = srv.add_insecure_port("[::]:0")
    add_OperationServiceServicer_to_server(
        MockOperationService([op1, op2, op3, op4, op5, op6]),
        srv,
    )
    await srv.start()

    channel = None
    try:
        channel = Channel(
            domain=f"localhost:{port}",
            options=[(INSECURE, True)],
            credentials=NoCredentials(),
        )
        operation = Operation(".nebius.common.v1.OperationService.Get", channel, op0)

        assert operation.progress_tracker() is None

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert tracker.description() == ""
        assert tracker.started_at() is None
        assert tracker.estimated_finished_at() is None
        assert tracker.work_fraction() is None
        assert tracker.time_fraction() is None
        assert tracker.steps() == []

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert tracker.description() == "phase-1"
        assert tracker.started_at() == to_local(started_now)
        assert tracker.estimated_finished_at() is None
        assert tracker.work_fraction() == pytest.approx(0.2)
        assert tracker.time_fraction() is None
        assert tracker.steps() == []

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert tracker.description() == "phase-1-est"
        assert tracker.started_at() == to_local(started_future)
        assert tracker.estimated_finished_at() == to_local(estimate_future)
        assert tracker.work_fraction() is None
        assert tracker.time_fraction() == 0.0

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert tracker.description() == "phase-2"
        assert tracker.estimated_finished_at() == to_local(estimate_past)
        assert tracker.time_fraction() == 1.0
        assert tracker.work_fraction() is None
        steps = tracker.steps()
        assert len(steps) == 2
        assert steps[0].description() == "step-a"
        assert steps[0].work_fraction() == pytest.approx(0.25)
        assert steps[1].description() == "step-b"
        assert steps[1].work_fraction() is None

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert tracker.description() == "phase-2 updated"
        assert tracker.estimated_finished_at() == to_local(estimate_updated)
        assert tracker.work_fraction() == pytest.approx(0.7)
        time_fraction = tracker.time_fraction()
        assert time_fraction is not None
        assert 0.0 <= time_fraction <= 1.0
        steps = tracker.steps()
        assert len(steps) == 2
        assert steps[0].work_fraction() == pytest.approx(1.0)
        assert steps[0].finished_at() is not None
        assert steps[1].description() == "step-b updated"
        assert steps[1].work_fraction() == pytest.approx(0.5)

        await operation.update()
        tracker = operation.progress_tracker()
        assert tracker is not None
        assert operation.done() is True
        assert tracker.description() == "done"
        assert tracker.work_fraction() == 1.0
        assert tracker.time_fraction() == 1.0
        assert tracker.estimated_finished_at() == to_local(finished_time)
        assert operation.finished_at == to_local(finished_time)
        steps = tracker.steps()
        assert len(steps) == 2
        assert steps[0].work_fraction() == pytest.approx(1.0)
        assert steps[1].work_fraction() == pytest.approx(1.0)
    finally:
        if channel is not None:
            await channel.close()
        await srv.stop(0)


@pytest.mark.asyncio
async def test_operation_progress_tracker_mlflow_cluster_operation() -> None:
    import nebius.api.nebius.common.v1alpha1.operation_pb2 as operation_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.aio.operation import Operation
    from nebius.base.options import INSECURE

    channel = None
    try:
        channel = Channel(
            domain="localhost",
            options=[(INSECURE, True)],
            credentials=NoCredentials(),
        )
        op = operation_pb2.Operation(id="mlflow-op-1")
        operation = Operation(
            ".nebius.msp.mlflow.v1alpha1.ClusterService.Create",
            channel,
            op,
        )
        assert operation.progress_tracker() is None
    finally:
        if channel is not None:
            await channel.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/capacity/v1/resource_advice.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(nebius/capacity/v1/resource_advice.proto\x12\x12nebius.capacity.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\"\xf5\x01\n\x0eResourceAdvice\x12Q\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x11\xbaH\x03\xc8\x01\x01\xe2J\x08\x12\x06tenantR\x08metadata\x12\x42\n\x04spec\x18\x02 \x01(\x0b\x32&.nebius.capacity.v1.ResourceAdviceSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12\x46\n\x06status\x18\x03 \x01(\x0b\x32(.nebius.capacity.v1.ResourceAdviceStatusB\x04\xbaJ\x01\x05R\x06status:\x04\xbaJ\x01\x02\"\xbd\x01\n\x12ResourceAdviceSpec\x12\x1c\n\x06region\x18\x01 \x01(\tB\x04\xbaJ\x01\x02R\x06region\x12\x1c\n\x06\x66\x61\x62ric\x18\x02 \x01(\tB\x04\xbaJ\x01\x02R\x06\x66\x61\x62ric\x12W\n\x10\x63ompute_instance\x18\n \x01(\x0b\x32*.nebius.capacity.v1.ComputeInstanceDetailsH\x00R\x0f\x63omputeInstanceB\x12\n\x10resource_details\"\xab\x03\n\x16\x43omputeInstanceDetails\x12 \n\x08platform\x18\x01 \x01(\tB\x04\xbaJ\x01\x02R\x08platform\x12O\n\x06preset\x18\x02 \x01(\x0b\x32\x31.nebius.capacity.v1.ComputeInstanceDetails.PresetB\x04\xbaJ\x01\x02R\x06preset\x12\x30\n\x14gpu_memory_gigabytes\x18\x03 \x01(\x05R\x12gpuMemoryGigabytes\x1a\xeb\x01\n\x06Preset\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12Y\n\tresources\x18\x02 \x01(\x0b\x32;.nebius.capacity.v1.ComputeInstanceDetails.Preset.ResourcesR\tresources\x1ar\n\tResources\x12\x1d\n\nvcpu_count\x18\x01 \x01(\x05R\tvcpuCount\x12)\n\x10memory_gibibytes\x18\x02 \x01(\x05R\x0fmemoryGibibytes\x12\x1b\n\tgpu_count\x18\x03 \x01(\x05R\x08gpuCount\"\xe7\x04\n\x14ResourceAdviceStatus\x12Q\n\x08reserved\x18\x01 \x01(\x0b\x32\x35.nebius.capacity.v1.ResourceAdviceStatus.AvailabilityR\x08reserved\x12R\n\ton_demand\x18\x02 \x01(\x0b\x32\x35.nebius.capacity.v1.ResourceAdviceStatus.AvailabilityR\x08onDemand\x12W\n\x0bpreemptible\x18\x03 \x01(\x0b\x32\x35.nebius.capacity.v1.ResourceAdviceStatus.AvailabilityR\x0bpreemptible\x1a\xce\x02\n\x0c\x41vailability\x12^\n\ndata_state\x18\x01 \x01(\x0e\x32?.nebius.capacity.v1.ResourceAdviceStatus.Availability.DataStateR\tdataState\x12\x1c\n\tavailable\x18\x02 \x01(\rR\tavailable\x12\x14\n\x05limit\x18\x04 \x01(\rR\x05limit\x12=\n\x0c\x65\x66\x66\x65\x63tive_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0b\x65\x66\x66\x65\x63tiveAt\"k\n\tDataState\x12\x1a\n\x16\x44\x41TA_STATE_UNSPECIFIED\x10\x00\x12\x14\n\x10\x44\x41TA_STATE_FRESH\x10\x01\x12\x14\n\x10\x44\x41TA_STATE_STALE\x10\x02\x12\x16\n\x12\x44\x41TA_STATE_UNKNOWN\x10\nBd\n\x19\x61i.nebius.pub.capacity.v1B\x13ResourceAdviceProtoP\x01Z0github.com/nebius/gosdk/proto/nebius/capacity/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.capacity.v1.resource_advice_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031ai.nebius.pub.capacity.v1B\023ResourceAdviceProtoP\001Z0github.com/nebius/gosdk/proto/nebius/capacity/v1'
  _RESOURCEADVICE.fields_by_name['metadata']._options = None
  _RESOURCEADVICE.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001\342J\010\022\006tenant'
  _RESOURCEADVICE.fields_by_name['spec']._options = None
  _RESOURCEADVICE.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _RESOURCEADVICE.fields_by_name['status']._options = None
  _RESOURCEADVICE.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _RESOURCEADVICE._options = None
  _RESOURCEADVICE._serialized_options = b'\272J\001\002'
  _RESOURCEADVICESPEC.fields_by_name['region']._options = None
  _RESOURCEADVICESPEC.fields_by_name['region']._serialized_options = b'\272J\001\002'
  _RESOURCEADVICESPEC.fields_by_name['fabric']._options = None
  _RESOURCEADVICESPEC.fields_by_name['fabric']._serialized_options = b'\272J\001\002'
  _COMPUTEINSTANCEDETAILS.fields_by_name['platform']._options = None
  _COMPUTEINSTANCEDETAILS.fields_by_name['platform']._serialized_options = b'\272J\001\002'
  _COMPUTEINSTANCEDETAILS.fields_by_name['preset']._options = None
  _COMPUTEINSTANCEDETAILS.fields_by_name['preset']._serialized_options = b'\272J\001\002'
  _globals['_RESOURCEADVICE']._serialized_start=186
  _globals['_RESOURCEADVICE']._serialized_end=431
  _globals['_RESOURCEADVICESPEC']._serialized_start=434
  _globals['_RESOURCEADVICESPEC']._serialized_end=623
  _globals['_COMPUTEINSTANCEDETAILS']._serialized_start=626
  _globals['_COMPUTEINSTANCEDETAILS']._serialized_end=1053
  _globals['_COMPUTEINSTANCEDETAILS_PRESET']._serialized_start=818
  _globals['_COMPUTEINSTANCEDETAILS_PRESET']._serialized_end=1053
  _globals['_COMPUTEINSTANCEDETAILS_PRESET_RESOURCES']._serialized_start=939
  _globals['_COMPUTEINSTANCEDETAILS_PRESET_RESOURCES']._serialized_end=1053
  _globals['_RESOURCEADVICESTATUS']._serialized_start=1056
  _globals['_RESOURCEADVICESTATUS']._serialized_end=1671
  _globals['_RESOURCEADVICESTATUS_AVAILABILITY']._serialized_start=1337
  _globals['_RESOURCEADVICESTATUS_AVAILABILITY']._serialized_end=1671
  _globals['_RESOURCEADVICESTATUS_AVAILABILITY_DATASTATE']._serialized_start=1564
  _globals['_RESOURCEADVICESTATUS_AVAILABILITY_DATASTATE']._serialized_end=1671
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1/route_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.vpc.v1 import route_pb2 as nebius_dot_vpc_dot_v1_dot_route__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/vpc/v1/route_service.proto\x12\rnebius.vpc.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x19nebius/vpc/v1/route.proto\"6\n\x0fGetRouteRequest\x12#\n\x02id\x18\x01 \x01(\tB\x13\xbaH\x03\xc8\x01\x01\xe2J\n\n\x08vpcrouteR\x02id\"j\n\x15GetRouteByNameRequest\x12\x35\n\tparent_id\x18\x01 \x01(\tB\x18\xbaH\x03\xc8\x01\x01\xe2J\x0f\n\rvpcroutetableR\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\x86\x01\n\x11ListRoutesRequest\x12\x35\n\tparent_id\x18\x01 \x01(\tB\x18\xbaH\x03\xc8\x01\x01\xe2J\x0f\n\rvpcroutetableR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"h\n\x12ListRoutesResponse\x12*\n\x05items\x18\x01 \x03(\x0b\x32\x14.nebius.vpc.v1.RouteR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xe9\x02\n\x12\x43reateRouteRequest\x12\x9c\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xdb\x01\xbaH\xc5\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xc8\x01\x01\xe2J\x0f\x12\rvpcroutetableR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32\x18.nebius.vpc.v1.RouteSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"\xde\x02\n\x12UpdateRouteRequest\x12\x99\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xd8\x01\xbaH\xc2\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xe2J\x0f\x12\rvpcroutetableR\x08metadata\x12,\n\x04spec\x18\x02 \x01(\x0b\x32\x18.nebius.vpc.v1.RouteSpecR\x04spec\"9\n\x12\x44\x65leteRouteRequest\x12#\n\x02id\x18\x01 \x01(\tB\x13\xbaH\x03\xc8\x01\x01\xe2J\n\n\x08vpcrouteR\x02id2\xc7\x03\n\x0cRouteService\x12;\n\x03Get\x12\x1e.nebius.vpc.v1.GetRouteRequest\x1a\x14.nebius.vpc.v1.Route\x12G\n\tGetByName\x12$.nebius.vpc.v1.GetRouteByNameRequest\x1a\x14.nebius.vpc.v1.Route\x12K\n\x04List\x12 .nebius.vpc.v1.ListRoutesRequest\x1a!.nebius.vpc.v1.ListRoutesResponse\x12H\n\x06\x43reate\x12!.nebius.vpc.v1.CreateRouteRequest\x1a\x1b.nebius.common.v1.Operation\x12H\n\x06Update\x12!.nebius.vpc.v1.UpdateRouteRequest\x1a\x1b.nebius.common.v1.Operation\x12H\n\x06\x44\x65lete\x12!.nebius.vpc.v1.DeleteRouteRequest\x1a\x1b.nebius.common.v1.Operation\x1a\x06\xbaJ\x03vpcBX\n\x14\x61i.nebius.pub.vpc.v1B\x11RouteServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/vpc/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1.route_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.vpc.v1B\021RouteServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/vpc/v1'
  _GETROUTEREQUEST.fields_by_name['id']._options = None
  _GETROUTEREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\342J\n\n\010vpcroute'
  _GETROUTEBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETROUTEBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\017\n\rvpcroutetable'
  _GETROUTEBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETROUTEBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTROUTESREQUEST.fields_by_name['parent_id']._options = None
  _LISTROUTESREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\017\n\rvpcroutetable'
  _CREATEROUTEREQUEST.fields_by_name['metadata']._options = None
  _CREATEROUTEREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\305\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\310\001\001\342J\017\022\rvpcroutetable'
  _CREATEROUTEREQUEST.fields_by_name['spec']._options = None
  _CREATEROUTEREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _UPDATEROUTEREQUEST.fields_by_name['metadata']._options = None
  _UPDATEROUTEREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\302\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\342J\017\022\rvpcroutetable'
  _DELETEROUTEREQUEST.fields_by_name['id']._options = None
  _DELETEROUTEREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\342J\n\n\010vpcroute'
  _ROUTESERVICE._options = None
  _ROUTESERVICE._serialized_options = b'\272J\003vpc'
  _globals['_GETROUTEREQUEST']._serialized_start=201
  _globals['_GETROUTEREQUEST']._serialized_end=255
  _globals['_GETROUTEBYNAMEREQUEST']._serialized_start=257
  _globals['_GETROUTEBYNAMEREQUEST']._serialized_end=363
  _globals['_LISTROUTESREQUEST']._serialized_start=366
  _globals['_LISTROUTESREQUEST']._serialized_end=500
  _globals['_LISTROUTESRESPONSE']._serialized_start=502
  _globals['_LISTROUTESRESPONSE']._serialized_end=606
  _globals['_CREATEROUTEREQUEST']._serialized_start=609
  _globals['_CREATEROUTEREQUEST']._serialized_end=970
  _globals['_UPDATEROUTEREQUEST']._serialized_start=973
  _globals['_UPDATEROUTEREQUEST']._serialized_end=1323
  _globals['_DELETEROUTEREQUEST']._serialized_start=1325
  _globals['_DELETEROUTEREQUEST']._serialized_end=1382
  _globals['_ROUTESERVICE']._serialized_start=1385
  _globals['_ROUTESERVICE']._serialized_end=1840
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/capacity/v1/resource_advice_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.capacity.v1 import resource_advice_pb2 as nebius_dot_capacity_dot_v1_dot_resource__advice__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0nebius/capacity/v1/resource_advice_service.proto\x12\x12nebius.capacity.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a(nebius/capacity/v1/resource_advice.proto\"\x87\x01\n\x19ListResourceAdviceRequest\x12.\n\tparent_id\x18\x01 \x01(\tB\x11\xbaH\x03\xc8\x01\x01\xe2J\x08\n\x06tenantR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"~\n\x1aListResourceAdviceResponse\x12\x38\n\x05items\x18\x01 \x03(\x0b\x32\".nebius.capacity.v1.ResourceAdviceR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\x9f\x01\n\x15ResourceAdviceService\x12\x65\n\x04List\x12-.nebius.capacity.v1.ListResourceAdviceRequest\x1a..nebius.capacity.v1.ListResourceAdviceResponse\x1a\x1f\xbaJ\x1c\x63\x61pacity-advisor.billing-cplBk\n\x19\x61i.nebius.pub.capacity.v1B\x1aResourceAdviceServiceProtoP\x01Z0github.com/nebius/gosdk/proto/nebius/capacity/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.capacity.v1.resource_advice_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031ai.nebius.pub.capacity.v1B\032ResourceAdviceServiceProtoP\001Z0github.com/nebius/gosdk/proto/nebius/capacity/v1'
  _LISTRESOURCEADVICEREQUEST.fields_by_name['parent_id']._options = None
  _LISTRESOURCEADVICEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\010\n\006tenant'
  _RESOURCEADVICESERVICE._options = None
  _RESOURCEADVICESERVICE._serialized_options = b'\272J\034capacity-advisor.billing-cpl'
  _globals['_LISTRESOURCEADVICEREQUEST']._serialized_start=170
  _globals['_LISTRESOURCEADVICEREQUEST']._serialized_end=305
  _globals['_LISTRESOURCEADVICERESPONSE']._serialized_start=307
  _globals['_LISTRESOURCEADVICERESPONSE']._serialized_end=433
  _globals['_RESOURCEADVICESERVICE']._serialized_start=436
  _globals['_RESOURCEADVICESERVICE']._serialized_end=595
# @@protoc_insertion_point(module_scope)



"""Federation bearer authentication utilities.

This module provides functions to perform OAuth 2.0 authorization code flow
with PKCE for federation bearer token authentication. It handles browser-based
authentication, callback server management, and token exchange.
"""

import asyncio
import ssl
import sys
import urllib.parse
import webbrowser
from logging import getLogger
from typing import TextIO

import aiohttp
from attr import dataclass

from nebius.base.tls_certificates import get_system_certificates

from .constants import AUTH_ENDPOINT, TOKEN_ENDPOINT
from .is_wsl import is_wsl
from .pkce import PKCE
from .server import CallbackHandler

log = getLogger(__name__)


@dataclass
class GetTokenResult:
    """Result of token retrieval.

    :ivar access_token: The access token string.
    :type access_token: str
    :ivar expires_in: Token expiration time in seconds, or None if not provided.
    :type expires_in: int or None
    """

    access_token: str
    expires_in: int | None = None


def https_url(raw_url: str) -> str:
    """Ensure the URL uses HTTPS scheme.

    :param raw_url: The input URL string.
    :type raw_url: str
    :return: The URL with HTTPS scheme.
    :rtype: str
    """
    if not raw_url.startswith(("http://", "https://")):
        return f"https://{raw_url}"
    return raw_url


async def open_browser(url: str) -> None:
    """Open the given URL in the default web browser.

    Handles special case for WSL environments by using cmd.exe.

    :param url: The URL to open.
    :type url: str
    """
    if sys.platform.startswith("linux") and is_wsl():
        import subprocess

        subprocess.run(  # noqa: S603
            ["cmd.exe", "/c", "start", url.replace("&", "^&")],  # noqa: S607
            check=True,
        )
    else:
        webbrowser.open(url)
    return None


async def get_code(
    client_id: str,
    auth_endpoint: str,
    federation_id: str,
    pkce_code: PKCE,
    writer: TextIO | None = None,
    no_browser_open: bool = False,
    timeout: float | None = 300,
) -> tuple[str, str]:
    """Obtain authorization code via OAuth callback.

    Starts a local callback server, constructs the authorization URL with PKCE,
    opens the browser (or prints the URL), and waits for the callback.

    :param client_id: The OAuth client ID.
    :type client_id: str
    :param auth_endpoint: The authorization endpoint URL.
    :type auth_endpoint: str
    :param federation_id: The federation identifier.
    :type federation_id: str
    :param pkce_code: The PKCE code challenge and method.
    :type pkce_code: :class:`PKCE`
    :param writer: Optional text stream to write messages to.
    :type writer: :class:`TextIO` or None
    :param no_browser_open: If True, do not open browser automatically.
    :type no_browser_open: bool
    :param timeout: Timeout in seconds for waiting for the code.
    :type timeout: float or None
    :return: A tuple of (authorization_code, redirect_uri).
    :rtype: tuple[str, str]
    :raises RuntimeError: If browser fails to open or no code received.
    :raises TimeoutError: If timeout waiting for code.
    """
    auth_url = urllib.parse.urlparse(auth_endpoint)
    callback = CallbackHandler()
    await callback.listen_and_serve()

    redirect_uri = callback.addr

    params = {
        "response_type": "code",
        "client_id": client_id,
        "scope": "openid",
        "state": callback.state,
        "redirect_uri": redirect_uri,
        "code_challenge": pkce_code.challenge,
        "code_challenge_method": pkce_code.method,
    }
    if federation_id:
        params["federation-id"] = federation_id

    query = urllib.parse.urlencode(params)
    full_auth_url = auth_url._replace(query=query).geturl()

    if no_browser_open:
        msg = (
            "To complete the authentication process, open this link in your"
            f" browser:\n{full_auth_url}"
        )
        log.debug(
            "Browser won't be opened. Show link to user.", extra={"url": full_auth_url}
        )
        if writer:
            print(msg, file=writer)
        else:
            log.info(msg)
    else:
        msg = (
            "Switch to your browser to complete login. If it didn't open "
            f"automatically, use:\n{full_auth_url}"
        )
        log.debug("Attempting to open browser.", extra={"url": full_auth_url})
        if writer:
            print(msg, file=writer)
        else:
            log.info(msg)
        try:
            await open_browser(full_auth_url)
        except Exception as browser_err:
            await callback.shutdown()
            raise RuntimeError(f"Failed to open browser: {browser_err}")

    try:
        code = await asyncio.wait_for(callback.wait_for_code(), timeout=timeout)
        if not code:
            raise RuntimeError("No code received from the callback")
        return code, redirect_uri
    except asyncio.TimeoutError:
        raise TimeoutError("Timeout waiting for auth code")
    finally:
        await callback.shutdown()


async def get_token(
    client_id: str,
    token_url: str,
    code: str,
    redirect_uri: str,
    verifier: str,
    ssl_ctx: ssl.SSLContext | None = None,
) -> GetTokenResult:
    """Exchange authorization code for access token.

    Sends a POST request to the token endpoint with the code and PKCE verifier.

    :param client_id: The OAuth client ID.
    :type client_id: str
    :param token_url: The token endpoint URL.
    :type token_url: str
    :param code: The authorization code received from callback.
    :type code: str
    :param redirect_uri: The redirect URI used in the request.
    :type redirect_uri: str
    :param verifier: The PKCE code verifier.
    :type verifier: str
    :param ssl_ctx: Optional SSL context for the request.
    :type ssl_ctx: :class:`ssl.SSLContext` or None
    :return: The token result containing access token and expiration.
    :rtype: :class:`GetTokenResult`
    :raises RuntimeError: If token request fails or response is invalid.
    """
    data = {
        "grant_type": "authorization_code",
        "code": code,
        "redirect_uri": redirect_uri,
        "client_id": client_id,
        "code_verifier": verifier,
    }
    if ssl_ctx is None:
        root_ca = get_system_certificates()
        ssl_ctx = ssl.create_default_context(cafile=root_ca)

    async with aiohttp.ClientSession() as session:
        async with session.post(
            token_url,
            data=data,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            ssl=ssl_ctx,
        ) as resp:
            if resp.status != 200:
                body = await resp.text()
                raise RuntimeError(f"Token request failed: {resp.status} {body}")
            ret = await resp.json()
            if not isinstance(ret, dict):
                raise RuntimeError(f"Unexpected response format: {ret}, expected dict")
            tok = ret.get("access_token", "")  # type: ignore[unused-ignore]
            expires_in = ret.get("expires_in", None)  # type: ignore[unused-ignore]
            if not isinstance(tok, str):
                raise RuntimeError(
                    f"Invalid token response: {ret}, expected 'access_token' as str"
                )
            if expires_in is not None and not isinstance(expires_in, int):
                raise RuntimeError(
                    f"Invalid token response: {ret}, expected 'expires_in' as int or"
                    " None"
                )
            return GetTokenResult(access_token=tok, expires_in=expires_in)


async def authorize(
    client_id: str,
    federation_endpoint: str,
    federation_id: str,
    writer: TextIO | None = None,
    no_browser_open: bool = False,
    timeout: float | None = 300,
    ssl_ctx: ssl.SSLContext | None = None,
) -> GetTokenResult:
    """Perform full OAuth authorization flow and return access token.

    Combines getting the authorization code and exchanging it for a token.

    :param client_id: The OAuth client ID.
    :type client_id: str
    :param federation_endpoint: The base federation endpoint URL.
    :type federation_endpoint: str
    :param federation_id: The federation identifier.
    :type federation_id: str
    :param writer: Optional text stream to write messages to.
    :type writer: :class:`TextIO` or None
    :param no_browser_open: If True, do not open browser automatically.
    :type no_browser_open: bool
    :param timeout: Timeout in seconds for the entire flow.
    :type timeout: float or None
    :param ssl_ctx: Optional SSL context for token request.
    :type ssl_ctx: :class:`ssl.SSLContext` or None
    :return: The token result containing access token and expiration.
    :rtype: :class:`GetTokenResult`
    """
    token_url = urllib.parse.urljoin(https_url(federation_endpoint), TOKEN_ENDPOINT)
    auth_url = urllib.parse.urljoin(https_url(federation_endpoint), AUTH_ENDPOINT)

    pkce = PKCE()

    code, redirect_uri = await get_code(
        client_id=client_id,
        auth_endpoint=auth_url,
        federation_id=federation_id,
        pkce_code=pkce,
        writer=writer,
        no_browser_open=no_browser_open,
        timeout=timeout,
    )

    log.debug("Auth code received")

    token = await get_token(
        client_id=client_id,
        token_url=token_url,
        code=code,
        redirect_uri=redirect_uri,
        verifier=pkce.verifier,
        ssl_ctx=ssl_ctx,
    )

    log.debug("Access token received")
    return token



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Warnings(_message.Message):
    __slots__ = ["warnings"]
    WARNINGS_FIELD_NUMBER: _ClassVar[int]
    warnings: _containers.RepeatedCompositeFieldContainer[Warning]
    def __init__(self, warnings: _Optional[_Iterable[_Union[Warning, _Mapping]]] = ...) -> None: ...

class Warning(_message.Message):
    __slots__ = ["target", "code", "summary", "summary_fallback", "details", "details_fallback", "path"]
    class Target(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        TARGET_UNSPECIFIED: _ClassVar[Warning.Target]
        TARGET_CLI: _ClassVar[Warning.Target]
        TARGET_TF: _ClassVar[Warning.Target]
        TARGET_CONSOLE: _ClassVar[Warning.Target]
    TARGET_UNSPECIFIED: Warning.Target
    TARGET_CLI: Warning.Target
    TARGET_TF: Warning.Target
    TARGET_CONSOLE: Warning.Target
    class Code(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        CODE_UNSPECIFIED: _ClassVar[Warning.Code]
        CODE_REGION_ROUTING_FAILOVER: _ClassVar[Warning.Code]
        CODE_DEPRECATED_TOOL_VERSION: _ClassVar[Warning.Code]
        CODE_DEPRECATED_ENDPOINT: _ClassVar[Warning.Code]
        CODE_DEPRECATED_PROTO: _ClassVar[Warning.Code]
        CODE_DEPRECATED_SPEC_VALUE_REQUEST: _ClassVar[Warning.Code]
        CODE_DEPRECATED_SPEC_VALUE_RESPONSE: _ClassVar[Warning.Code]
    CODE_UNSPECIFIED: Warning.Code
    CODE_REGION_ROUTING_FAILOVER: Warning.Code
    CODE_DEPRECATED_TOOL_VERSION: Warning.Code
    CODE_DEPRECATED_ENDPOINT: Warning.Code
    CODE_DEPRECATED_PROTO: Warning.Code
    CODE_DEPRECATED_SPEC_VALUE_REQUEST: Warning.Code
    CODE_DEPRECATED_SPEC_VALUE_RESPONSE: Warning.Code
    TARGET_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_FALLBACK_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FIELD_NUMBER: _ClassVar[int]
    DETAILS_FALLBACK_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    target: Warning.Target
    code: Warning.Code
    summary: str
    summary_fallback: str
    details: str
    details_fallback: str
    path: str
    def __init__(self, target: _Optional[_Union[Warning.Target, str]] = ..., code: _Optional[_Union[Warning.Code, str]] = ..., summary: _Optional[str] = ..., summary_fallback: _Optional[str] = ..., details: _Optional[str] = ..., details_fallback: _Optional[str] = ..., path: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/capacity/v1/capacity_block_group_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.capacity.v1 import capacity_block_group_pb2 as nebius_dot_capacity_dot_v1_dot_capacity__block__group__pb2
from nebius.api.nebius.capacity.v1 import resource_affinity_pb2 as nebius_dot_capacity_dot_v1_dot_resource__affinity__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5nebius/capacity/v1/capacity_block_group_service.proto\x12\x12nebius.capacity.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a-nebius/capacity/v1/capacity_block_group.proto\x1a*nebius/capacity/v1/resource_affinity.proto\"Q\n\x1cGetCapacityBlockGroupRequest\x12\x31\n\x02id\x18\x01 \x01(\tB!\xbaH\x03\xc8\x01\x01\xbaJ\x01\x03\xe2J\x14\n\x12\x63\x61pacityblockgroupR\x02id\"\xdb\x01\n.GetCapacityBlockGroupByResourceAffinityRequest\x12.\n\tparent_id\x18\x01 \x01(\tB\x11\xbaH\x03\xc8\x01\x01\xe2J\x08\n\x06tenantR\x08parentId\x12\x1e\n\x06region\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x06region\x12Y\n\x11resource_affinity\x18\x03 \x01(\x0b\x32$.nebius.capacity.v1.ResourceAffinityB\x06\xbaH\x03\xc8\x01\x01R\x10resourceAffinity\"\x98\x01\n\x1eListCapacityBlockGroupsRequest\x12.\n\tparent_id\x18\x01 \x01(\tB\x11\xbaH\x03\xc8\x01\x01\xe2J\x08\n\x06tenantR\x08parentId\x12\'\n\tpage_size\x18\x02 \x01(\x03\x42\n\xbaH\x07\"\x05\x18\xc8\x01(\x00R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x87\x01\n\x1fListCapacityBlockGroupsResponse\x12<\n\x05items\x18\x01 \x03(\x0b\x32&.nebius.capacity.v1.CapacityBlockGroupR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"[\n&ListCapacityBlockGroupResourcesRequest\x12\x31\n\x02id\x18\x01 \x01(\tB!\xbaH\x03\xc8\x01\x01\xbaJ\x01\x03\xe2J\x14\n\x12\x63\x61pacityblockgroupR\x02id\"L\n\'ListCapacityBlockGroupResourcesResponse\x12!\n\x0cresource_ids\x18\x01 \x03(\tR\x0bresourceIds2\x9e\x04\n\x19\x43\x61pacityBlockGroupService\x12_\n\x03Get\x12\x30.nebius.capacity.v1.GetCapacityBlockGroupRequest\x1a&.nebius.capacity.v1.CapacityBlockGroup\x12\x83\x01\n\x15GetByResourceAffinity\x12\x42.nebius.capacity.v1.GetCapacityBlockGroupByResourceAffinityRequest\x1a&.nebius.capacity.v1.CapacityBlockGroup\x12o\n\x04List\x12\x32.nebius.capacity.v1.ListCapacityBlockGroupsRequest\x1a\x33.nebius.capacity.v1.ListCapacityBlockGroupsResponse\x12\x88\x01\n\rListResources\x12:.nebius.capacity.v1.ListCapacityBlockGroupResourcesRequest\x1a;.nebius.capacity.v1.ListCapacityBlockGroupResourcesResponse\x1a\x1e\xbaJ\x1b\x63\x61pacity-blocks.billing-cplBo\n\x19\x61i.nebius.pub.capacity.v1B\x1e\x43\x61pacityBlockGroupServiceProtoP\x01Z0github.com/nebius/gosdk/proto/nebius/capacity/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.capacity.v1.capacity_block_group_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031ai.nebius.pub.capacity.v1B\036CapacityBlockGroupServiceProtoP\001Z0github.com/nebius/gosdk/proto/nebius/capacity/v1'
  _GETCAPACITYBLOCKGROUPREQUEST.fields_by_name['id']._options = None
  _GETCAPACITYBLOCKGROUPREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\272J\001\003\342J\024\n\022capacityblockgroup'
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['parent_id']._options = None
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\010\n\006tenant'
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['region']._options = None
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['region']._serialized_options = b'\272H\003\310\001\001'
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['resource_affinity']._options = None
  _GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST.fields_by_name['resource_affinity']._serialized_options = b'\272H\003\310\001\001'
  _LISTCAPACITYBLOCKGROUPSREQUEST.fields_by_name['parent_id']._options = None
  _LISTCAPACITYBLOCKGROUPSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\010\n\006tenant'
  _LISTCAPACITYBLOCKGROUPSREQUEST.fields_by_name['page_size']._options = None
  _LISTCAPACITYBLOCKGROUPSREQUEST.fields_by_name['page_size']._serialized_options = b'\272H\007\"\005\030\310\001(\000'
  _LISTCAPACITYBLOCKGROUPRESOURCESREQUEST.fields_by_name['id']._options = None
  _LISTCAPACITYBLOCKGROUPRESOURCESREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\272J\001\003\342J\024\n\022capacityblockgroup'
  _CAPACITYBLOCKGROUPSERVICE._options = None
  _CAPACITYBLOCKGROUPSERVICE._serialized_options = b'\272J\033capacity-blocks.billing-cpl'
  _globals['_GETCAPACITYBLOCKGROUPREQUEST']._serialized_start=223
  _globals['_GETCAPACITYBLOCKGROUPREQUEST']._serialized_end=304
  _globals['_GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST']._serialized_start=307
  _globals['_GETCAPACITYBLOCKGROUPBYRESOURCEAFFINITYREQUEST']._serialized_end=526
  _globals['_LISTCAPACITYBLOCKGROUPSREQUEST']._serialized_start=529
  _globals['_LISTCAPACITYBLOCKGROUPSREQUEST']._serialized_end=681
  _globals['_LISTCAPACITYBLOCKGROUPSRESPONSE']._serialized_start=684
  _globals['_LISTCAPACITYBLOCKGROUPSRESPONSE']._serialized_end=819
  _globals['_LISTCAPACITYBLOCKGROUPRESOURCESREQUEST']._serialized_start=821
  _globals['_LISTCAPACITYBLOCKGROUPRESOURCESREQUEST']._serialized_end=912
  _globals['_LISTCAPACITYBLOCKGROUPRESOURCESRESPONSE']._serialized_start=914
  _globals['_LISTCAPACITYBLOCKGROUPRESOURCESRESPONSE']._serialized_end=990
  _globals['_CAPACITYBLOCKGROUPSERVICE']._serialized_start=993
  _globals['_CAPACITYBLOCKGROUPSERVICE']._serialized_end=1535
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1/security_group_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.vpc.v1 import security_group_pb2 as nebius_dot_vpc_dot_v1_dot_security__group__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*nebius/vpc/v1/security_group_service.proto\x12\rnebius.vpc.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\"nebius/vpc/v1/security_group.proto\"F\n\x17GetSecurityGroupRequest\x12+\n\x02id\x18\x01 \x01(\tB\x1b\xbaH\x03\xc8\x01\x01\xe2J\x12\n\x10vpcsecuritygroupR\x02id\"l\n\x1dGetSecurityGroupByNameRequest\x12/\n\tparent_id\x18\x01 \x01(\tB\x12\xbaH\x03\xc8\x01\x01\xe2J\t\n\x07projectR\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\x88\x01\n\x19ListSecurityGroupsRequest\x12/\n\tparent_id\x18\x01 \x01(\tB\x12\xbaH\x03\xc8\x01\x01\xe2J\t\n\x07projectR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\x96\x01\n\"ListSecurityGroupsByNetworkRequest\x12\x34\n\nnetwork_id\x18\x01 \x01(\tB\x15\xbaH\x03\xc8\x01\x01\xe2J\x0c\n\nvpcnetworkR\tnetworkId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"x\n\x1aListSecurityGroupsResponse\x12\x32\n\x05items\x18\x01 \x03(\x0b\x32\x1c.nebius.vpc.v1.SecurityGroupR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xf3\x02\n\x1a\x43reateSecurityGroupRequest\x12\x96\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xd5\x01\xbaH\xc5\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xc8\x01\x01\xe2J\t\x12\x07projectR\x08metadata\x12<\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.vpc.v1.SecurityGroupSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"\xe8\x02\n\x1aUpdateSecurityGroupRequest\x12\x93\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xd2\x01\xbaH\xc2\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xe2J\t\x12\x07projectR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.vpc.v1.SecurityGroupSpecR\x04spec\"I\n\x1a\x44\x65leteSecurityGroupRequest\x12+\n\x02id\x18\x01 \x01(\tB\x1b\xbaH\x03\xc8\x01\x01\xe2J\x12\n\x10vpcsecuritygroupR\x02id2\x86\x05\n\x14SecurityGroupService\x12K\n\x03Get\x12&.nebius.vpc.v1.GetSecurityGroupRequest\x1a\x1c.nebius.vpc.v1.SecurityGroup\x12W\n\tGetByName\x12,.nebius.vpc.v1.GetSecurityGroupByNameRequest\x1a\x1c.nebius.vpc.v1.SecurityGroup\x12[\n\x04List\x12(.nebius.vpc.v1.ListSecurityGroupsRequest\x1a).nebius.vpc.v1.ListSecurityGroupsResponse\x12m\n\rListByNetwork\x12\x31.nebius.vpc.v1.ListSecurityGroupsByNetworkRequest\x1a).nebius.vpc.v1.ListSecurityGroupsResponse\x12P\n\x06\x43reate\x12).nebius.vpc.v1.CreateSecurityGroupRequest\x1a\x1b.nebius.common.v1.Operation\x12P\n\x06Update\x12).nebius.vpc.v1.UpdateSecurityGroupRequest\x1a\x1b.nebius.common.v1.Operation\x12P\n\x06\x44\x65lete\x12).nebius.vpc.v1.DeleteSecurityGroupRequest\x1a\x1b.nebius.common.v1.Operation\x1a\x06\xbaJ\x03vpcB`\n\x14\x61i.nebius.pub.vpc.v1B\x19SecurityGroupServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/vpc/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1.security_group_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.vpc.v1B\031SecurityGroupServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/vpc/v1'
  _GETSECURITYGROUPREQUEST.fields_by_name['id']._options = None
  _GETSECURITYGROUPREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\342J\022\n\020vpcsecuritygroup'
  _GETSECURITYGROUPBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETSECURITYGROUPBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\t\n\007project'
  _GETSECURITYGROUPBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETSECURITYGROUPBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTSECURITYGROUPSREQUEST.fields_by_name['parent_id']._options = None
  _LISTSECURITYGROUPSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001\342J\t\n\007project'
  _LISTSECURITYGROUPSBYNETWORKREQUEST.fields_by_name['network_id']._options = None
  _LISTSECURITYGROUPSBYNETWORKREQUEST.fields_by_name['network_id']._serialized_options = b'\272H\003\310\001\001\342J\014\n\nvpcnetwork'
  _CREATESECURITYGROUPREQUEST.fields_by_name['metadata']._options = None
  _CREATESECURITYGROUPREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\305\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\310\001\001\342J\t\022\007project'
  _CREATESECURITYGROUPREQUEST.fields_by_name['spec']._options = None
  _CREATESECURITYGROUPREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _UPDATESECURITYGROUPREQUEST.fields_by_name['metadata']._options = None
  _UPDATESECURITYGROUPREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\302\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\342J\t\022\007project'
  _DELETESECURITYGROUPREQUEST.fields_by_name['id']._options = None
  _DELETESECURITYGROUPREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001\342J\022\n\020vpcsecuritygroup'
  _SECURITYGROUPSERVICE._options = None
  _SECURITYGROUPSERVICE._serialized_options = b'\272J\003vpc'
  _globals['_GETSECURITYGROUPREQUEST']._serialized_start=219
  _globals['_GETSECURITYGROUPREQUEST']._serialized_end=289
  _globals['_GETSECURITYGROUPBYNAMEREQUEST']._serialized_start=291
  _globals['_GETSECURITYGROUPBYNAMEREQUEST']._serialized_end=399
  _globals['_LISTSECURITYGROUPSREQUEST']._serialized_start=402
  _globals['_LISTSECURITYGROUPSREQUEST']._serialized_end=538
  _globals['_LISTSECURITYGROUPSBYNETWORKREQUEST']._serialized_start=541
  _globals['_LISTSECURITYGROUPSBYNETWORKREQUEST']._serialized_end=691
  _globals['_LISTSECURITYGROUPSRESPONSE']._serialized_start=693
  _globals['_LISTSECURITYGROUPSRESPONSE']._serialized_end=813
  _globals['_CREATESECURITYGROUPREQUEST']._serialized_start=816
  _globals['_CREATESECURITYGROUPREQUEST']._serialized_end=1187
  _globals['_UPDATESECURITYGROUPREQUEST']._serialized_start=1190
  _globals['_UPDATESECURITYGROUPREQUEST']._serialized_end=1550
  _globals['_DELETESECURITYGROUPREQUEST']._serialized_start=1552
  _globals['_DELETESECURITYGROUPREQUEST']._serialized_end=1625
  _globals['_SECURITYGROUPSERVICE']._serialized_start=1628
  _globals['_SECURITYGROUPSERVICE']._serialized_end=2274
# @@protoc_insertion_point(module_scope)



"""Service descriptor utilities for gRPC services.

This module provides classes and functions to extract service names from gRPC
stub classes and handle service method descriptors in the Nebius async SDK.
It includes stub implementations that prevent actual calls and an extractor
channel for introspecting service metadata.
"""

from typing import Any, Protocol, TypeVar

from google.protobuf.message import Message
from grpc import (
    CallCredentials,
    ChannelConnectivity,
    Compression,
)
from grpc.aio import Channel as GRPCChannel
from grpc.aio._base_call import (
    StreamStreamCall,
    StreamUnaryCall,
    UnaryStreamCall,
    UnaryUnaryCall,
)
from grpc.aio._base_channel import (
    StreamStreamMultiCallable,
    StreamUnaryMultiCallable,
    UnaryStreamMultiCallable,
    UnaryUnaryMultiCallable,
)
from grpc.aio._typing import (
    DeserializingFunction,
    MetadataType,
    RequestIterableType,
    SerializingFunction,
)

from nebius.base.error import SDKError
from nebius.base.methods import service_from_method_name

Req = TypeVar("Req", bound=Message)
Res = TypeVar("Res", bound=Message)


class NotATrueCallError(SDKError):
    """Error raised when attempting to call a stub method that is not meant to be
    executed.

    This exception is used by stub classes in this module to indicate that they
    are placeholders for service introspection, not actual callable methods.
    """

    def __init__(self, *args: object) -> None:
        super().__init__("This class is not meant to be run as a call.")


class NoMethodsInServiceError(SDKError):
    """Error raised when no methods are found in a service stub.

    This exception occurs during service name extraction if the stub class
    does not define any gRPC methods.
    """

    def __init__(self, *args: object) -> None:
        super().__init__("No methods found in service stub")


class StubUU(UnaryUnaryMultiCallable):  # type: ignore[unused-ignore,misc,type-arg]
    """Stub implementation for unary-unary gRPC methods.

    This class raises NotATrueCallError when called, as it is used only for
    service introspection, not actual RPC execution.
    """

    def __call__(  # type: ignore
        self,
        request,  # type: ignore[unused-ignore]
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubUS(UnaryStreamMultiCallable):  # type: ignore[unused-ignore,misc,type-arg]
    """Stub implementation for unary-stream gRPC methods.

    This class raises NotATrueCallError when called, as it is used only for
    service introspection, not actual RPC execution.
    """

    def __call__(  # type: ignore
        self,
        request,  # type: ignore[unused-ignore]
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryStreamCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubSU(StreamUnaryMultiCallable):  # type: ignore[unused-ignore,misc]
    """Stub implementation for stream-unary gRPC methods.

    This class raises NotATrueCallError when called, as it is used only for
    service introspection, not actual RPC execution.
    """

    def __call__(  # type: ignore[unused-ignore]
        self,
        request_iterator: RequestIterableType | None = None,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamUnaryCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubSS(StreamStreamMultiCallable):  # type: ignore[unused-ignore,misc]
    """Stub implementation for stream-stream gRPC methods.

    This class raises NotATrueCallError when called, as it is used only for
    service introspection, not actual RPC execution.
    """

    def __call__(  # type: ignore[unused-ignore]
        self,
        request_iterator: RequestIterableType | None = None,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamStreamCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class ExtractorChannel(GRPCChannel):  # type: ignore[unused-ignore,misc]
    """A mock gRPC channel for extracting service names from stub classes.

    This channel implementation records the last method called on it and can
    extract the service name from the method name. It is used to introspect
    gRPC stub classes without making actual network calls.
    """

    def __init__(self) -> None:
        super().__init__()
        self._last_method = ""

    def get_service_name(self) -> str:
        """Extract the service name from the last recorded method.

        :return: The service name.
        :rtype: str
        :raises NoMethodsInServiceError: If no methods have been recorded.
        """
        if self._last_method == "":
            raise NoMethodsInServiceError()
        return service_from_method_name(self._last_method)

    def unary_unary(  # type: ignore[unused-ignore, override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = False,
    ) -> UnaryUnaryMultiCallable[Req, Res]:  # type: ignore[unused-ignore, override]
        """Record a unary-unary method call and return a stub.

        :param method: The method name.
        :type method: str
        :param request_serializer: Optional request serializer.
        :type request_serializer: :class:`SerializingFunction` or None
        :param response_deserializer: Optional response deserializer.
        :type response_deserializer: :class:`DeserializingFunction` or None
        :param _registered_method: Whether the method is registered.
        :type _registered_method: bool or None
        :return: A stub callable.
        :rtype: :class:`UnaryUnaryMultiCallable`
        """
        self._last_method = method
        return StubUU()

    async def close(self, grace: float | None = None) -> None:
        """Close the channel (no-op for this mock implementation).

        :param grace: Optional grace period.
        :type grace: float or None
        """
        pass

    async def __aenter__(self) -> "ExtractorChannel":
        """Enter async context.

        :return: Self.
        :rtype: :class:`ExtractorChannel`
        """
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        """Exit async context.

        :param exc_type: Exception type.
        :type exc_type: Any
        :param exc_val: Exception value.
        :type exc_val: Any
        :param exc_tb: Exception traceback.
        :type exc_tb: Any
        """
        await self.close(None)

    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity:
        """Get the channel state (always READY for this mock).

        :param try_to_connect: Whether to attempt connection.
        :type try_to_connect: bool
        :return: The connectivity state.
        :rtype: :class:`ChannelConnectivity`
        """
        return ChannelConnectivity.READY

    async def wait_for_state_change(
        self,
        last_observed_state: ChannelConnectivity,
    ) -> None:
        """Wait for state change (not implemented for this mock).

        :param last_observed_state: The last observed state.
        :type last_observed_state: :class:`ChannelConnectivity`
        :raises NotImplementedError: Always raised.
        """
        raise NotImplementedError("this method has no meaning for this channel")

    async def channel_ready(self) -> None:
        """Wait for channel to be ready (no-op for this mock)."""
        return

    def unary_stream(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> UnaryStreamMultiCallable[Req, Res]:  # type: ignore[unused-ignore]
        """Record a unary-stream method call and return a stub.

        :param method: The method name.
        :type method: str
        :param request_serializer: Optional request serializer.
        :type request_serializer: :class:`SerializingFunction` or None
        :param response_deserializer: Optional response deserializer.
        :type response_deserializer: :class:`DeserializingFunction` or None
        :param _registered_method: Whether the method is registered.
        :type _registered_method: bool or None
        :return: A stub callable.
        :rtype: :class:`UnaryStreamMultiCallable`
        """
        self._last_method = method
        return StubUS()

    def stream_unary(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> StreamUnaryMultiCallable:
        """Record a stream-unary method call and return a stub.

        :param method: The method name.
        :type method: str
        :param request_serializer: Optional request serializer.
        :type request_serializer: :class:`SerializingFunction` or None
        :param response_deserializer: Optional response deserializer.
        :type response_deserializer: :class:`DeserializingFunction` or None
        :param _registered_method: Whether the method is registered.
        :type _registered_method: bool or None
        :return: A stub callable.
        :rtype: :class:`StreamUnaryMultiCallable`
        """
        self._last_method = method
        return StubSU()

    def stream_stream(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> StreamStreamMultiCallable:
        """Record a stream-stream method call and return a stub.

        :param method: The method name.
        :type method: str
        :param request_serializer: Optional request serializer.
        :type request_serializer: :class:`SerializingFunction` or None
        :param response_deserializer: Optional response deserializer.
        :type response_deserializer: :class:`DeserializingFunction` or None
        :param _registered_method: Whether the method is registered.
        :type _registered_method: bool or None
        :return: A stub callable.
        :rtype: :class:`StreamStreamMultiCallable`
        """
        self._last_method = method
        return StubSS()


class ServiceStub(Protocol):
    """Protocol for gRPC service stub classes.

    This protocol defines the expected interface for gRPC stub classes
    that can be instantiated with a channel.
    """

    def __init__(self, channel: GRPCChannel) -> None: ...


def from_stub_class(stub: type[ServiceStub]) -> str:
    """Extract the service name from a gRPC stub class.

    Uses an ExtractorChannel to instantiate the stub and record method calls,
    then extracts the service name from the recorded method.

    :param stub: The stub class to extract from.
    :type stub: type[:class:`ServiceStub`]
    :return: The service name.
    :rtype: str
    """
    if hasattr(stub, "__PB2_NAME__"):
        return getattr(stub, "__PB2_NAME__")  # type: ignore[no-any-return]
    extractor = ExtractorChannel()
    _ = stub(extractor)
    ret = extractor.get_service_name()
    setattr(stub, "__PB2_NAME__", ret)
    return ret



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/cors.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cnebius/storage/v1/cors.proto\x12\x11nebius.storage.v1\x1a\x1b\x62uf/validate/validate.proto\"P\n\x11\x43ORSConfiguration\x12;\n\x05rules\x18\x01 \x03(\x0b\x32\x1b.nebius.storage.v1.CORSRuleB\x08\xbaH\x05\x92\x01\x02\x10\x64R\x05rules\"\x99\x02\n\x08\x43ORSRule\x12\x13\n\x02id\x18\x01 \x01(\tH\x00R\x02id\x88\x01\x01\x12\'\n\x0f\x61llowed_headers\x18\x02 \x03(\tR\x0e\x61llowedHeaders\x12/\n\x0f\x61llowed_origins\x18\x03 \x03(\tB\x06\xbaH\x03\xc8\x01\x01R\x0e\x61llowedOrigins\x12/\n\x0f\x61llowed_methods\x18\x04 \x03(\tB\x06\xbaH\x03\xc8\x01\x01R\x0e\x61llowedMethods\x12%\n\x0e\x65xpose_headers\x18\x05 \x03(\tR\rexposeHeaders\x12+\n\x0fmax_age_seconds\x18\x06 \x01(\x05H\x01R\rmaxAgeSeconds\x88\x01\x01\x42\x05\n\x03_idB\x12\n\x10_max_age_secondsBX\n\x18\x61i.nebius.pub.storage.v1B\tCorsProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.cors_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\tCorsProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _CORSCONFIGURATION.fields_by_name['rules']._options = None
  _CORSCONFIGURATION.fields_by_name['rules']._serialized_options = b'\272H\005\222\001\002\020d'
  _CORSRULE.fields_by_name['allowed_origins']._options = None
  _CORSRULE.fields_by_name['allowed_origins']._serialized_options = b'\272H\003\310\001\001'
  _CORSRULE.fields_by_name['allowed_methods']._options = None
  _CORSRULE.fields_by_name['allowed_methods']._serialized_options = b'\272H\003\310\001\001'
  _globals['_CORSCONFIGURATION']._serialized_start=80
  _globals['_CORSCONFIGURATION']._serialized_end=160
  _globals['_CORSRULE']._serialized_start=163
  _globals['_CORSRULE']._serialized_end=444
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class ExportFormat(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    EXPORT_FORMAT_UNSPECIFIED: _ClassVar[ExportFormat]
    EXPORT_FORMAT_FOCUS_1_2_CSV: _ClassVar[ExportFormat]
EXPORT_FORMAT_UNSPECIFIED: ExportFormat
EXPORT_FORMAT_FOCUS_1_2_CSV: ExportFormat



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/common/v1/warning.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1enebius/common/v1/warning.proto\x12\x10nebius.common.v1\"A\n\x08Warnings\x12\x35\n\x08warnings\x18\x01 \x03(\x0b\x32\x19.nebius.common.v1.WarningR\x08warnings\"\xd7\x04\n\x07Warning\x12\x38\n\x06target\x18\x01 \x01(\x0e\x32 .nebius.common.v1.Warning.TargetR\x06target\x12\x32\n\x04\x63ode\x18\x02 \x01(\x0e\x32\x1e.nebius.common.v1.Warning.CodeR\x04\x63ode\x12\x18\n\x07summary\x18\x03 \x01(\tR\x07summary\x12)\n\x10summary_fallback\x18\x04 \x01(\tR\x0fsummaryFallback\x12\x18\n\x07\x64\x65tails\x18\x05 \x01(\tR\x07\x64\x65tails\x12)\n\x10\x64\x65tails_fallback\x18\x06 \x01(\tR\x0f\x64\x65tailsFallback\x12\x12\n\x04path\x18\x07 \x01(\tR\x04path\"S\n\x06Target\x12\x16\n\x12TARGET_UNSPECIFIED\x10\x00\x12\x0e\n\nTARGET_CLI\x10\x01\x12\r\n\tTARGET_TF\x10\x02\x12\x12\n\x0eTARGET_CONSOLE\x10\x03\"\xea\x01\n\x04\x43ode\x12\x14\n\x10\x43ODE_UNSPECIFIED\x10\x00\x12 \n\x1c\x43ODE_REGION_ROUTING_FAILOVER\x10\x01\x12 \n\x1c\x43ODE_DEPRECATED_TOOL_VERSION\x10\x02\x12\x1c\n\x18\x43ODE_DEPRECATED_ENDPOINT\x10\x03\x12\x19\n\x15\x43ODE_DEPRECATED_PROTO\x10\x04\x12&\n\"CODE_DEPRECATED_SPEC_VALUE_REQUEST\x10\x05\x12\'\n#CODE_DEPRECATED_SPEC_VALUE_RESPONSE\x10\x06\x42Y\n\x17\x61i.nebius.pub.common.v1B\x0cWarningProtoP\x01Z.github.com/nebius/gosdk/proto/nebius/common/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.common.v1.warning_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027ai.nebius.pub.common.v1B\014WarningProtoP\001Z.github.com/nebius/gosdk/proto/nebius/common/v1'
  _globals['_WARNINGS']._serialized_start=52
  _globals['_WARNINGS']._serialized_end=117
  _globals['_WARNING']._serialized_start=120
  _globals['_WARNING']._serialized_end=719
  _globals['_WARNING_TARGET']._serialized_start=399
  _globals['_WARNING_TARGET']._serialized_end=482
  _globals['_WARNING_CODE']._serialized_start=485
  _globals['_WARNING_CODE']._serialized_end=719
# @@protoc_insertion_point(module_scope)
